@@ -28,16 +28,20 @@
 static size_t chunk_buf_sz = 8192;
 static chunk *chunks, *chunks_oversized;
 static chunk *chunk_buffers;
+static int chunks_oversized_n;
 static const array *chunkqueue_default_tempdirs = NULL;
 static off_t chunkqueue_default_tempfile_size = DEFAULT_TEMPFILE_SIZE;
 
 void chunkqueue_set_chunk_size (size_t sz)
 {
-    chunk_buf_sz = sz > 0 ? ((sz + 1023) & ~1023uL) : 8192;
+    size_t x = 1024;
+    while (x < sz && x < (1u << 30)) x <<= 1;
+    chunk_buf_sz = sz > 0 ? x : 8192;
 }
 
 void chunkqueue_set_tempdirs_default_reset (void)
 {
+    chunk_buf_sz = 8192;
     chunkqueue_default_tempdirs = NULL;
     chunkqueue_default_tempfile_size = DEFAULT_TEMPFILE_SIZE;
 }
@@ -131,15 +135,50 @@ static void chunk_free(chunk *c) {
 	free(c);
 }
 
-buffer * chunk_buffer_acquire(void) {
+static chunk * chunk_pop_oversized(size_t sz) {
+    /* future: might have buckets of certain sizes, up to socket buf sizes */
+    if (chunks_oversized && chunks_oversized->mem->size >= sz) {
+        --chunks_oversized_n;
+        chunk *c = chunks_oversized;
+        chunks_oversized = c->next;
+        return c;
+    }
+    return NULL;
+}
+
+static void chunk_push_oversized(chunk * const c, const size_t sz) {
+    if (chunks_oversized_n < 64 && chunk_buf_sz >= 4096) {
+        ++chunks_oversized_n;
+        chunk **co = &chunks_oversized;
+        while (*co && sz < (*co)->mem->size) co = &(*co)->next;
+        c->next = *co;
+        *co = c;
+    }
+    else
+        chunk_free(c);
+}
+
+__attribute_returns_nonnull__
+static buffer * chunk_buffer_acquire_sz(size_t sz) {
     chunk *c;
     buffer *b;
-    if (chunks) {
-        c = chunks;
-        chunks = c->next;
+    if (sz <= chunk_buf_sz) {
+        if (chunks) {
+            c = chunks;
+            chunks = c->next;
+        }
+        else
+            c = chunk_init(chunk_buf_sz);
+            /* future: might choose to pop from chunks_oversized, if available
+             * (even if larger than sz) rather than allocating new chunk
+             * (and if doing so, might replace chunks_oversized_n) */
     }
     else {
-        c = chunk_init(chunk_buf_sz);
+        /*(round up to nearest chunk_buf_sz)*/
+        sz = (sz + (chunk_buf_sz-1)) & ~(chunk_buf_sz-1);
+        c = chunk_pop_oversized(sz);
+        if (NULL == c)
+            c = chunk_init(sz);
     }
     c->next = chunk_buffers;
     chunk_buffers = c;
@@ -148,21 +187,47 @@ buffer * chunk_buffer_acquire(void) {
     return b;
 }
 
+buffer * chunk_buffer_acquire(void) {
+    return chunk_buffer_acquire_sz(chunk_buf_sz);
+}
+
 void chunk_buffer_release(buffer *b) {
     if (NULL == b) return;
-    if (b->size >= chunk_buf_sz && chunk_buffers) {
+    if (chunk_buffers) {
         chunk *c = chunk_buffers;
         chunk_buffers = c->next;
         c->mem = b;
-        c->next = chunks;
-        chunks = c;
         buffer_clear(b);
+        if (b->size == chunk_buf_sz) {
+            c->next = chunks;
+            chunks = c;
+        }
+        else if (b->size > chunk_buf_sz)
+            chunk_push_oversized(c, b->size);
+        else
+            chunk_free(c);
     }
     else {
         buffer_free(b);
     }
 }
 
+size_t chunk_buffer_prepare_append(buffer * const b, size_t sz) {
+    if (sz > chunk_buffer_string_space(b)) {
+        sz += b->used ? b->used : 1;
+        buffer * const cb = chunk_buffer_acquire_sz(sz);
+        /* swap buffer contents and copy original b->ptr into larger b->ptr */
+        /*(this does more than buffer_move())*/
+        buffer tb = *b;
+        *b = *cb;
+        *cb = tb;
+        if ((b->used = tb.used))
+            memcpy(b->ptr, tb.ptr, tb.used);
+        chunk_buffer_release(cb);
+    }
+    return chunk_buffer_string_space(b);
+}
+
 __attribute_returns_nonnull__
 static chunk * chunk_acquire(size_t sz) {
     if (sz <= chunk_buf_sz) {
@@ -174,13 +239,10 @@ static chunk * chunk_acquire(size_t sz) {
         sz = chunk_buf_sz;
     }
     else {
-        sz = (sz + 8191) & ~8191uL;
-        /* future: might have buckets of certain sizes, up to socket buf sizes*/
-        if (chunks_oversized && chunks_oversized->mem->size >= sz) {
-            chunk *c = chunks_oversized;
-            chunks_oversized = c->next;
-            return c;
-        }
+        /*(round up to nearest chunk_buf_sz)*/
+        sz = (sz + (chunk_buf_sz-1)) & ~(chunk_buf_sz-1);
+        chunk *c = chunk_pop_oversized(sz);
+        if (c) return c;
     }
 
     return chunk_init(sz);
@@ -195,10 +257,7 @@ static void chunk_release(chunk *c) {
     }
     else if (sz > chunk_buf_sz) {
         chunk_reset(c);
-        chunk **co = &chunks_oversized;
-        while (*co && sz < (*co)->mem->size) co = &(*co)->next;
-        c->next = *co;
-        *co = c;
+        chunk_push_oversized(c, sz);
     }
     else {
         chunk_free(c);
@@ -217,6 +276,7 @@ void chunkqueue_chunk_pool_clear(void)
         chunk_free(c);
     }
     chunks_oversized = NULL;
+    chunks_oversized_n = 0;
 }
 
 void chunkqueue_chunk_pool_free(void)
@@ -825,16 +885,6 @@ void chunkqueue_compact_mem(chunkqueue *cq, size_t clen) {
 }
 
 static int chunk_open_file_chunk(chunk * const restrict c, log_error_st * const restrict errh) {
-	off_t offset, toSend;
-	struct stat st;
-
-	force_assert(NULL != c);
-	force_assert(FILE_CHUNK == c->type);
-	force_assert(c->offset >= 0 && c->offset <= c->file.length);
-
-	offset = c->offset;
-	toSend = c->file.length - c->offset;
-
 	if (-1 == c->file.fd) {
 		/* (permit symlinks; should already have been checked.  However, TOC-TOU remains) */
 		if (-1 == (c->file.fd = fdevent_open_cloexec(c->mem->ptr, 1, O_RDONLY, 0))) {
@@ -846,12 +896,18 @@ static int chunk_open_file_chunk(chunk * const restrict c, log_error_st * const
 	/*(skip file size checks if file is temp file created by lighttpd)*/
 	if (c->file.is_temp) return 0;
 
+	force_assert(FILE_CHUNK == c->type);
+	force_assert(c->offset >= 0 && c->offset <= c->file.length);
+
+	struct stat st;
 	if (-1 == fstat(c->file.fd, &st)) {
 		log_perror(errh, __FILE__, __LINE__, "fstat failed");
 		return -1;
 	}
 
-	if (offset > st.st_size || toSend > st.st_size || offset > st.st_size - toSend) {
+	const off_t offset = c->offset;
+	const off_t len = c->file.length - c->offset;
+	if (offset > st.st_size || len > st.st_size || offset > st.st_size - len) {
 		log_error(errh, __FILE__, __LINE__, "file shrunk: %s", c->mem->ptr);
 		return -1;
 	}
@@ -864,6 +920,165 @@ int chunkqueue_open_file_chunk(chunkqueue * const restrict cq, log_error_st * co
 }
 
 
+#if defined(HAVE_MMAP)
+__attribute_cold__
+#endif
+__attribute_noinline__
+static ssize_t
+chunkqueue_write_chunk_file_intermed (const int fd, chunk * const restrict c, log_error_st * const errh)
+{
+    char buf[16384];
+    char *data = buf;
+    const off_t count = c->file.length - c->offset;
+    uint32_t dlen = count < (off_t)sizeof(buf) ? (uint32_t)count : sizeof(buf);
+    chunkqueue cq = {c,c,0,0,0,0,0}; /*(fake cq for chunkqueue_peek_data())*/
+    if (0 != chunkqueue_peek_data(&cq, &data, &dlen, errh) && 0 == dlen)
+        return -1;
+    ssize_t wr;
+    do { wr = write(fd, data, dlen); } while (-1 == wr && errno == EINTR);
+    return wr;
+}
+
+
+#if defined(HAVE_MMAP)
+/*(improved from network_write_mmap.c)*/
+static off_t
+mmap_align_offset (off_t start)
+{
+    static off_t pagemask = 0;
+    if (0 == pagemask) {
+        long pagesize = sysconf(_SC_PAGESIZE);
+        if (-1 == pagesize) pagesize = 4096;
+        pagemask = ~((off_t)pagesize - 1); /* pagesize always power-of-2 */
+    }
+    return (start & pagemask);
+}
+#endif
+
+
+#if defined HAVE_SYS_SENDFILE_H && defined HAVE_SENDFILE \
+ && (!defined _LARGEFILE_SOURCE || defined HAVE_SENDFILE64) \
+ && defined(__linux__) && !defined HAVE_SENDFILE_BROKEN
+#include <sys/sendfile.h>
+#include <stdint.h>
+#endif
+static ssize_t
+chunkqueue_write_chunk_file (const int fd, chunk * const restrict c, log_error_st * const errh)
+{
+    /*(similar to network_write_file_chunk_mmap(), but does not use send() on
+    *  Windows because fd is expected to be file or pipe here, not socket)*/
+
+    if (0 != chunk_open_file_chunk(c, errh))
+        return -1;
+
+    const off_t count = c->file.length - c->offset;
+    if (0 == count) return 0; /*(sanity check)*/
+
+    ssize_t wr;
+  #if defined HAVE_SYS_SENDFILE_H && defined HAVE_SENDFILE \
+   && (!defined _LARGEFILE_SOURCE || defined HAVE_SENDFILE64) \
+   && defined(__linux__) && !defined HAVE_SENDFILE_BROKEN
+    /* Linux kernel >= 2.6.33 supports sendfile() between most fd types */
+    off_t offset = c->offset;
+    wr = sendfile(fd, c->file.fd, &offset, count<INT32_MAX ? count : INT32_MAX);
+    if (wr >= 0) return wr;
+
+    if (wr < 0 && (errno == EINVAL || errno == ENOSYS))
+  #endif
+    {
+      #if defined(HAVE_MMAP)
+        /*(caller is responsible for handling SIGBUS if chunkqueue might contain
+         * untrusted file, i.e. any file other than lighttpd-created tempfile)*/
+        /*(tempfiles are expected for input, MAP_PRIVATE used for portability)*/
+        /*(mmaps and writes complete chunk instead of only small parts; files
+         * are expected to be temp files with reasonable chunk sizes)*/
+
+        /* (re)mmap the buffer if range is not covered completely */
+        if (MAP_FAILED == c->file.mmap.start
+            || c->offset < c->file.mmap.offset
+            || c->file.length
+                 > (off_t)(c->file.mmap.offset + c->file.mmap.length)) {
+
+            if (MAP_FAILED != c->file.mmap.start) {
+                munmap(c->file.mmap.start, c->file.mmap.length);
+                c->file.mmap.start = MAP_FAILED;
+            }
+
+            c->file.mmap.offset = mmap_align_offset(c->offset);
+            c->file.mmap.length = c->file.length - c->file.mmap.offset;
+            c->file.mmap.start  =
+              mmap(NULL, c->file.mmap.length, PROT_READ, MAP_PRIVATE,
+                   c->file.fd, c->file.mmap.offset);
+
+          #if 0
+            /* close() fd as soon as fully mmap() rather than when done w/ chunk
+             * (possibly worthwhile to keep active fd count lower) */
+            if (c->file.is_temp && !c->file.refchg) {
+                close(c->file.fd);
+                c->file.fd = -1;
+            }
+          #endif
+        }
+
+        if (MAP_FAILED != c->file.mmap.start) {
+            const char * const data =
+              c->file.mmap.start + c->offset - c->file.mmap.offset;
+            do { wr = write(fd,data,count); } while (-1 == wr && errno==EINTR);
+        }
+        else
+      #endif
+            wr = chunkqueue_write_chunk_file_intermed(fd, c, errh);
+    }
+    return wr;
+}
+
+
+static ssize_t
+chunkqueue_write_chunk_mem (const int fd, const chunk * const restrict c)
+{
+    const void * const buf = c->mem->ptr + c->offset;
+    const size_t count = chunk_buffer_string_length(c->mem) - (size_t)c->offset;
+    ssize_t wr;
+    do { wr = write(fd, buf, count); } while (-1 == wr && errno == EINTR);
+    return wr;
+}
+
+
+ssize_t
+chunkqueue_write_chunk (const int fd, chunkqueue * const restrict cq, log_error_st * const restrict errh)
+{
+    /*(note: expects non-empty cq->first)*/
+    chunk * const c = cq->first;
+    switch (c->type) {
+      case MEM_CHUNK:
+        return chunkqueue_write_chunk_mem(fd, c);
+      case FILE_CHUNK:
+        return chunkqueue_write_chunk_file(fd, c, errh);
+      default:
+        errno = EINVAL;
+        return -1;
+    }
+}
+
+
+ssize_t
+chunkqueue_write_chunk_to_pipe (const int fd, chunkqueue * const restrict cq, log_error_st * const restrict errh)
+{
+    /*(note: expects non-empty cq->first)*/
+  #ifdef SPLICE_F_NONBLOCK /* splice() temp files to pipe on Linux */
+    chunk * const c = cq->first;
+    if (c->type == FILE_CHUNK) {
+        loff_t abs_offset = c->offset;
+        return (0 == chunk_open_file_chunk(c, errh))
+          ? splice(c->file.fd, &abs_offset, fd, NULL,
+                   (size_t)(c->file.length - c->offset), SPLICE_F_NONBLOCK)
+          : -1;
+    }
+  #endif
+    return chunkqueue_write_chunk(fd, cq, errh);
+}
+
+
 void
 chunkqueue_small_resp_optim (chunkqueue * const restrict cq)
 {
@@ -939,21 +1154,26 @@ chunkqueue_peek_data (chunkqueue * const cq,
           case FILE_CHUNK:
             if (c->file.fd >= 0 || 0 == chunk_open_file_chunk(c, errh)) {
                 off_t offset = c->offset;
-                off_t toSend = c->file.length - c->offset;
-                if (toSend > (off_t)space)
-                    toSend = (off_t)space;
+                off_t len = c->file.length - c->offset;
+                if (len > (off_t)space)
+                    len = (off_t)space;
 
                 if (-1 == lseek(c->file.fd, offset, SEEK_SET)) {
-                    log_perror(errh, __FILE__, __LINE__, "lseek");
+                    log_perror(errh, __FILE__, __LINE__, "lseek(\"%s\")",
+                               c->mem->ptr);
                     return -1;
                 }
-                toSend = read(c->file.fd, data_in + *dlen, (size_t)toSend);
-                if (toSend <= 0) { /* -1 error; 0 EOF (unexpected) */
-                    log_perror(errh, __FILE__, __LINE__, "read");
+                ssize_t rd;
+                do {
+                    rd = read(c->file.fd, data_in + *dlen, (size_t)len);
+                } while (-1 == rd && errno == EINTR);
+                if (rd <= 0) { /* -1 error; 0 EOF (unexpected) */
+                    log_perror(errh, __FILE__, __LINE__, "read(\"%s\")",
+                               c->mem->ptr);
                     return -1;
                 }
 
-                *dlen += (uint32_t)toSend;
+                *dlen += (uint32_t)rd;
                 break;
             }
             return -1;
