@@ -36,12 +36,10 @@ void config_get_config_cond_info(config_cond_info * const cfginfo, uint32_t idx)
     cfginfo->comp = dc->comp;
     cfginfo->cond = dc->cond;
     cfginfo->string = &dc->string;
-    cfginfo->comp_tag = dc->comp_tag;
     cfginfo->comp_key = dc->comp_key;
-    cfginfo->op = dc->op;
 }
 
-int config_plugin_value_tobool (data_unset *du, int default_value)
+int config_plugin_value_tobool (const data_unset *du, int default_value)
 {
     if (NULL == du) return default_value;
     if (du->type == TYPE_STRING) {
@@ -63,7 +61,7 @@ int config_plugin_value_tobool (data_unset *du, int default_value)
         return default_value;
 }
 
-int32_t config_plugin_value_to_int32 (data_unset *du, int32_t default_value)
+int32_t config_plugin_value_to_int32 (const data_unset *du, int32_t default_value)
 {
     if (NULL == du) return default_value;
     if (du->type == TYPE_STRING) {
@@ -86,7 +84,7 @@ int config_plugin_values_init_block(server * const srv, const array * const ca,
     int rc = 1; /* default is success */
 
     for (int i = 0; cpk[i].ktype != T_CONFIG_UNSET; ++i) {
-        data_unset * const du =
+        const data_unset * const du =
           array_get_element_klen(ca, cpk[i].k, cpk[i].klen);
         if (NULL == du) continue; /* not found */
 
@@ -287,7 +285,7 @@ int config_plugin_values_init(server * const srv, void *p_d, const config_plugin
 
         matches[n] = 0;
         for (int i = 0; cpk[i].ktype != T_CONFIG_UNSET; ++i) {
-            data_unset * const du =
+            const data_unset * const du =
               array_get_element_klen(ca, cpk[i].k, cpk[i].klen);
             if (NULL == du) continue; /* not found */
 
@@ -591,7 +589,8 @@ static cond_result_t config_check_cond_nocache(request_st * const r, const data_
 		break;
 
 	case COMP_HTTP_REQUEST_HEADER:
-		*((const buffer **)&l) = http_header_request_get(r, dc->ext, CONST_BUF_LEN(dc->comp_tag));
+		*((const buffer **)&l) =
+		  http_header_request_get(r, dc->ext, CONST_BUF_LEN(&dc->comp_tag));
 		if (NULL == l) l = (buffer *)&empty_string;
 		break;
 	case COMP_HTTP_REQUEST_METHOD:
@@ -605,12 +604,12 @@ static cond_result_t config_check_cond_nocache(request_st * const r, const data_
 
 	if (NULL == l) { /*(should not happen)*/
 		log_error(r->conf.errh, __FILE__, __LINE__,
-			"%s () compare to NULL", dc->comp_key->ptr);
+			"%s compare to NULL", dc->comp_key);
 		return COND_RESULT_FALSE;
 	}
 	else if (debug_cond) {
 		log_error(r->conf.errh, __FILE__, __LINE__,
-			"%s (%s) compare to %s", dc->comp_key->ptr, l->ptr, dc->string.ptr);
+			"%s compare to %s", dc->comp_key, l->ptr);
 	}
 
 	switch(dc->cond) {
