@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <string.h>
 
+__attribute_pure__
 static data_config * configparser_get_data_config(const array *a, const char *k, const size_t klen) {
   return (data_config *)array_get_data_unset(a, k, klen);
 }
@@ -154,8 +155,202 @@ static int configparser_remoteip_normalize_compat(buffer *rvalue) {
   return rc;
 }
 
+__attribute_pure__
+static comp_key_t
+configparser_comp_key_id(const buffer * const obj_tag, const buffer * const comp_tag)
+{
+  /* $REQUEST_HEADER["..."] */
+  /* $SERVER["socket"] */
+  /* $HTTP["..."] */
+  if (buffer_eq_slen(obj_tag, CONST_STR_LEN("REQUEST_HEADER")))
+    return COMP_HTTP_REQUEST_HEADER;
+  else if (buffer_eq_slen(obj_tag, CONST_STR_LEN("SERVER")))
+    return (buffer_eq_slen(comp_tag, CONST_STR_LEN("socket")))
+      ? COMP_SERVER_SOCKET
+      : COMP_UNSET;
+  else if (buffer_eq_slen(obj_tag, CONST_STR_LEN("HTTP"))) {
+    static const struct {
+      comp_key_t comp;
+      uint32_t len;
+      const char *comp_tag;
+    } comps[] = {
+      { COMP_HTTP_URL,            CONST_LEN_STR("url"           ) },
+      { COMP_HTTP_HOST,           CONST_LEN_STR("host"          ) },
+      { COMP_HTTP_REQUEST_HEADER, CONST_LEN_STR("referer"       ) },
+      { COMP_HTTP_USER_AGENT,     CONST_LEN_STR("useragent"     ) },
+      { COMP_HTTP_REQUEST_HEADER, CONST_LEN_STR("user-agent"    ) },
+      { COMP_HTTP_LANGUAGE,       CONST_LEN_STR("language"      ) },
+      { COMP_HTTP_REQUEST_HEADER, CONST_LEN_STR("cookie"        ) },
+      { COMP_HTTP_REMOTE_IP,      CONST_LEN_STR("remoteip"      ) },
+      { COMP_HTTP_REMOTE_IP,      CONST_LEN_STR("remote-ip"     ) },
+      { COMP_HTTP_QUERY_STRING,   CONST_LEN_STR("querystring"   ) },
+      { COMP_HTTP_QUERY_STRING,   CONST_LEN_STR("query-string"  ) },
+      { COMP_HTTP_REQUEST_METHOD, CONST_LEN_STR("request-method") },
+      { COMP_HTTP_SCHEME,         CONST_LEN_STR("scheme"        ) }
+    };
 
-#line 159 "./configparser.c"
+    for (uint32_t i = 0; i < sizeof(comps)/sizeof(comps[0]); ++i) {
+      if (buffer_eq_slen(comp_tag, comps[i].comp_tag, comps[i].len))
+        return comps[i].comp;
+    }
+  }
+  return COMP_UNSET;
+}
+
+static void
+configparser_parse_condition(config_t * const ctx, const buffer * const obj_tag, const buffer * const comp_tag, const config_cond_t cond, buffer * const rvalue)
+{
+    const char *op = NULL;
+    switch(cond) {
+    case CONFIG_COND_NE:      op = "!="; break;
+    case CONFIG_COND_EQ:      op = "=="; break;
+    case CONFIG_COND_NOMATCH: op = "!~"; break;
+    case CONFIG_COND_MATCH:   op = "=~"; break;
+    default:
+      force_assert(0);
+      return; /* unreachable */
+    }
+
+    buffer * const tb = ctx->srv->tmp_buf;
+    buffer_copy_buffer(tb, &ctx->current->key);
+    buffer_append_string_len(tb, CONST_STR_LEN(" / "));
+    const uint32_t comp_offset = buffer_string_length(tb);
+    buffer_append_string_len(tb, CONST_STR_LEN("$"));
+    buffer_append_string_buffer(tb, obj_tag); /*(HTTP, REQUEST_HEADER, SERVER)*/
+    buffer_append_string_len(tb, CONST_STR_LEN("[\""));
+    buffer_append_string_buffer(tb, comp_tag);
+    buffer_append_string_len(tb, CONST_STR_LEN("\"] "));
+    buffer_append_string_len(tb, op, 2);
+    buffer_append_string_len(tb, CONST_STR_LEN(" \""));
+    buffer_append_string_buffer(tb, rvalue);
+    buffer_append_string_len(tb, CONST_STR_LEN("\""));
+
+    data_config *dc;
+    if (NULL != (dc = configparser_get_data_config(ctx->all_configs,
+                                                   CONST_BUF_LEN(tb)))) {
+      configparser_push(ctx, dc, 0);
+    }
+    else {
+      dc = data_config_init();
+      dc->cond = cond;
+      dc->comp = configparser_comp_key_id(obj_tag, comp_tag);
+
+      buffer_copy_buffer(&dc->key, tb);
+      buffer_copy_buffer(&dc->comp_tag, comp_tag);
+      dc->comp_key = dc->key.ptr + comp_offset;
+
+      if (COMP_UNSET == dc->comp) {
+          fprintf(stderr, "error comp_key %s", dc->comp_key);
+          ctx->ok = 0;
+      }
+      else if (COMP_HTTP_LANGUAGE == dc->comp) {
+        dc->comp = COMP_HTTP_REQUEST_HEADER;
+        buffer_copy_string_len(&dc->comp_tag, CONST_STR_LEN("Accept-Language"));
+      }
+      else if (COMP_HTTP_USER_AGENT == dc->comp) {
+        dc->comp = COMP_HTTP_REQUEST_HEADER;
+        buffer_copy_string_len(&dc->comp_tag, CONST_STR_LEN("User-Agent"));
+      }
+      else if (COMP_HTTP_REMOTE_IP == dc->comp
+               && (dc->cond == CONFIG_COND_EQ || dc->cond == CONFIG_COND_NE)) {
+        char * const slash = strchr(rvalue->ptr, '/'); /* CIDR mask */
+        char * const colon = strchr(rvalue->ptr, ':'); /* IPv6 */
+        if (NULL != slash && slash == rvalue->ptr){/*(skip AF_UNIX /path/file)*/
+        }
+        else if (NULL != slash) {
+          char *nptr;
+          const unsigned long nm_bits = strtoul(slash + 1, &nptr, 10);
+          if (*nptr || 0 == nm_bits || nm_bits > (NULL != colon ? 128 : 32)) {
+            /*(also rejects (slash+1 == nptr) which results in nm_bits = 0)*/
+            fprintf(stderr, "invalid or missing netmask: %s\n", rvalue->ptr);
+            ctx->ok = 0;
+          }
+          else {
+            int rc;
+            buffer_string_set_length(rvalue, (size_t)(slash - rvalue->ptr)); /*(truncate)*/
+            rc = (NULL == colon)
+              ? http_request_host_normalize(rvalue, 0)
+              : configparser_remoteip_normalize_compat(rvalue);
+            buffer_append_string_len(rvalue, CONST_STR_LEN("/"));
+            buffer_append_int(rvalue, (int)nm_bits);
+            if (0 != rc) {
+              fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
+              ctx->ok = 0;
+            }
+          }
+        }
+        else {
+          int rc = (NULL == colon)
+            ? http_request_host_normalize(rvalue, 0)
+            : configparser_remoteip_normalize_compat(rvalue);
+          if (0 != rc) {
+            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
+            ctx->ok = 0;
+          }
+        }
+      }
+      else if (COMP_SERVER_SOCKET == dc->comp) {
+        /*(redundant with parsing in network.c; not actually required here)*/
+        if (rvalue->ptr[0] != ':' /*(network.c special-cases ":" and "[]")*/
+            && !(rvalue->ptr[0] == '[' && rvalue->ptr[1] == ']')) {
+          if (http_request_host_normalize(rvalue, 0)) {
+            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
+            ctx->ok = 0;
+          }
+        }
+      }
+      else if (COMP_HTTP_HOST == dc->comp) {
+        if (dc->cond == CONFIG_COND_EQ || dc->cond == CONFIG_COND_NE) {
+          if (http_request_host_normalize(rvalue, 0)) {
+            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
+            ctx->ok = 0;
+          }
+        }
+      }
+
+      if (COMP_HTTP_REQUEST_HEADER == dc->comp) {
+        dc->ext = http_header_hkey_get(CONST_BUF_LEN(&dc->comp_tag));
+      }
+
+      buffer_copy_buffer(&dc->string, rvalue);
+
+      if (ctx->ok) switch(dc->cond) {
+      case CONFIG_COND_NE:
+      case CONFIG_COND_EQ:
+        break;
+      case CONFIG_COND_NOMATCH:
+      case CONFIG_COND_MATCH: {
+        if (!data_config_pcre_compile(dc)) {
+          ctx->ok = 0;
+        }
+        break;
+      }
+      default:
+        fprintf(stderr, "unknown condition for %s\n", dc->comp_key);
+        ctx->ok = 0;
+        break;
+      }
+
+      if (ctx->ok)
+        configparser_push(ctx, dc, 1);
+      else
+        dc->fn->free((data_unset*) dc);
+    }
+}
+
+static void
+configparser_parse_else_condition(config_t * const ctx)
+{
+    data_config * const dc = data_config_init();
+    dc->cond = CONFIG_COND_ELSE;
+    buffer_copy_buffer(&dc->key, &ctx->current->key);
+    buffer_append_string_len(&dc->key, CONST_STR_LEN(" / "));
+    buffer_append_string_len(&dc->key, CONST_STR_LEN("else_tmp_token"));
+    configparser_push(ctx, dc, 1);
+}
+
+
+#line 354 "./configparser.c"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -574,44 +769,44 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 24:
     case 25:
     case 26:
-#line 191 "../../src/configparser.y"
+#line 386 "../../src/configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 579 "./configparser.c"
+#line 774 "./configparser.c"
       break;
     case 36:
-#line 182 "../../src/configparser.y"
+#line 377 "../../src/configparser.y"
 { if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
-#line 584 "./configparser.c"
+#line 779 "./configparser.c"
       break;
     case 37:
-#line 183 "../../src/configparser.y"
+#line 378 "../../src/configparser.y"
 { if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
-#line 589 "./configparser.c"
+#line 784 "./configparser.c"
       break;
     case 38:
-#line 184 "../../src/configparser.y"
+#line 379 "../../src/configparser.y"
 { if ((yypminor->yy91)) (yypminor->yy91)->fn->free((yypminor->yy91)); }
-#line 594 "./configparser.c"
+#line 789 "./configparser.c"
       break;
     case 41:
-#line 185 "../../src/configparser.y"
+#line 380 "../../src/configparser.y"
 { array_free((yypminor->yy42)); }
-#line 599 "./configparser.c"
+#line 794 "./configparser.c"
       break;
     case 42:
-#line 186 "../../src/configparser.y"
+#line 381 "../../src/configparser.y"
 { array_free((yypminor->yy42)); }
-#line 604 "./configparser.c"
+#line 799 "./configparser.c"
       break;
     case 43:
-#line 187 "../../src/configparser.y"
+#line 382 "../../src/configparser.y"
 { buffer_free((yypminor->yy29)); }
-#line 609 "./configparser.c"
+#line 804 "./configparser.c"
       break;
     case 44:
-#line 188 "../../src/configparser.y"
+#line 383 "../../src/configparser.y"
 { buffer_free((yypminor->yy29)); }
-#line 614 "./configparser.c"
+#line 809 "./configparser.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -888,9 +1083,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 164 "../../src/configparser.y"
+#line 359 "../../src/configparser.y"
 { yymsp[-1].minor.yy18 = NULL; }
-#line 893 "./configparser.c"
+#line 1088 "./configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -903,7 +1098,7 @@ static void yy_reduce(
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 9:
-#line 193 "../../src/configparser.y"
+#line 388 "../../src/configparser.y"
 {
   if (ctx->ok) {
     buffer_copy_buffer(&yymsp[0].minor.yy91->key, yymsp[-2].minor.yy29);
@@ -927,11 +1122,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 930 "./configparser.c"
+#line 1125 "./configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 217 "../../src/configparser.y"
+#line 412 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (strncmp(yymsp[-2].minor.yy29->ptr, "env.", sizeof("env.") - 1) == 0) {
@@ -950,11 +1145,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 953 "./configparser.c"
+#line 1148 "./configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 236 "../../src/configparser.y"
+#line 431 "../../src/configparser.y"
 {
   if (ctx->ok) {
     array *vars = ctx->current->value;
@@ -985,11 +1180,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 988 "./configparser.c"
+#line 1183 "./configparser.c"
   yy_destructor(4,&yymsp[-1].minor);
         break;
       case 12:
-#line 267 "../../src/configparser.y"
+#line 462 "../../src/configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     yygotominor.yy29 = buffer_init_string("var.");
@@ -1001,10 +1196,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1004 "./configparser.c"
+#line 1199 "./configparser.c"
         break;
       case 13:
-#line 279 "../../src/configparser.y"
+#line 474 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1019,19 +1214,19 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1022 "./configparser.c"
+#line 1217 "./configparser.c"
   yy_destructor(6,&yymsp[-1].minor);
         break;
       case 14:
-#line 294 "../../src/configparser.y"
+#line 489 "../../src/configparser.y"
 {
   yygotominor.yy91 = yymsp[0].minor.yy91;
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1031 "./configparser.c"
+#line 1226 "./configparser.c"
         break;
       case 15:
-#line 299 "../../src/configparser.y"
+#line 494 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1056,10 +1251,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1059 "./configparser.c"
+#line 1254 "./configparser.c"
         break;
       case 16:
-#line 324 "../../src/configparser.y"
+#line 519 "../../src/configparser.y"
 {
   yygotominor.yy91 = (data_unset *)data_string_init();
   /* assumes data_string_init() result does not require swap and buffer_free()*/
@@ -1067,10 +1262,10 @@ static void yy_reduce(
   free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1070 "./configparser.c"
+#line 1265 "./configparser.c"
         break;
       case 17:
-#line 332 "../../src/configparser.y"
+#line 527 "../../src/configparser.y"
 {
   char *endptr;
   yygotominor.yy91 = (data_unset *)data_integer_init();
@@ -1085,10 +1280,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1088 "./configparser.c"
+#line 1283 "./configparser.c"
         break;
       case 18:
-#line 346 "../../src/configparser.y"
+#line 541 "../../src/configparser.y"
 {
   yygotominor.yy91 = (data_unset *)data_array_init();
   /* assumes data_array_init() result does not require swap and array_free() */
@@ -1096,29 +1291,29 @@ static void yy_reduce(
   free(yymsp[0].minor.yy42);
   yymsp[0].minor.yy42 = NULL;
 }
-#line 1099 "./configparser.c"
+#line 1294 "./configparser.c"
         break;
       case 19:
-#line 353 "../../src/configparser.y"
+#line 548 "../../src/configparser.y"
 {
   yygotominor.yy42 = array_init(8);
 }
-#line 1106 "./configparser.c"
+#line 1301 "./configparser.c"
   yy_destructor(9,&yymsp[-1].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 20:
-#line 356 "../../src/configparser.y"
+#line 551 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1116 "./configparser.c"
+#line 1311 "./configparser.c"
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 21:
-#line 361 "../../src/configparser.y"
+#line 556 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1140,20 +1335,20 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1143 "./configparser.c"
+#line 1338 "./configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 22:
-#line 383 "../../src/configparser.y"
+#line 578 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1152 "./configparser.c"
+#line 1347 "./configparser.c"
   yy_destructor(11,&yymsp[0].minor);
         break;
       case 23:
-#line 388 "../../src/configparser.y"
+#line 583 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1164,18 +1359,18 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1167 "./configparser.c"
+#line 1362 "./configparser.c"
         break;
       case 24:
-#line 399 "../../src/configparser.y"
+#line 594 "../../src/configparser.y"
 {
   yygotominor.yy91 = yymsp[0].minor.yy91;
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1175 "./configparser.c"
+#line 1370 "./configparser.c"
         break;
       case 25:
-#line 403 "../../src/configparser.y"
+#line 598 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1189,7 +1384,7 @@ static void yy_reduce(
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
 }
-#line 1192 "./configparser.c"
+#line 1387 "./configparser.c"
   yy_destructor(12,&yymsp[-1].minor);
         break;
       case 26:
@@ -1198,31 +1393,31 @@ static void yy_reduce(
       case 27:
         break;
       case 28:
-#line 420 "../../src/configparser.y"
+#line 615 "../../src/configparser.y"
 {
   data_config *dc;
   dc = configparser_get_data_config(ctx->srv->config_context, CONST_STR_LEN("global"));
   force_assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1208 "./configparser.c"
+#line 1403 "./configparser.c"
   yy_destructor(13,&yymsp[0].minor);
         break;
       case 29:
-#line 427 "../../src/configparser.y"
+#line 622 "../../src/configparser.y"
 {
   force_assert(ctx->current);
   configparser_pop(ctx);
   force_assert(ctx->current);
 }
-#line 1218 "./configparser.c"
+#line 1413 "./configparser.c"
         /* No destructor defined for globalstart */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 30:
-#line 433 "../../src/configparser.y"
+#line 628 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1241,12 +1436,12 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1244 "./configparser.c"
+#line 1439 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 31:
-#line 452 "../../src/configparser.y"
+#line 647 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1265,10 +1460,11 @@ static void yy_reduce(
     dc = (data_config *)array_extract_element_klen(ctx->all_configs, CONST_BUF_LEN(&yymsp[0].minor.yy18->key));
     force_assert(yymsp[0].minor.yy18 == dc);
     buffer_copy_buffer(&yymsp[0].minor.yy18->key, &yymsp[-3].minor.yy18->key);
+    yymsp[0].minor.yy18->comp_key = yymsp[0].minor.yy18->key.ptr + (yymsp[-3].minor.yy18->comp_key - yymsp[-3].minor.yy18->key.ptr);
     yymsp[0].minor.yy18->comp = yymsp[-3].minor.yy18->comp;
-    /*buffer_copy_buffer(yymsp[0].minor.yy18->comp_key, yymsp[-3].minor.yy18->comp_key);*/
     /*buffer_copy_buffer(&yymsp[0].minor.yy18->string, &yymsp[-3].minor.yy18->string);*/
-    pos = buffer_string_length(&yymsp[0].minor.yy18->key)-buffer_string_length(&yymsp[-3].minor.yy18->string)-2;
+    /* -2 for "==" and minus 3 for spaces and quotes around string (in key) */
+    pos = buffer_string_length(&yymsp[0].minor.yy18->key)-buffer_string_length(&yymsp[-3].minor.yy18->string)-5;
     switch(yymsp[-3].minor.yy18->cond) {
     case CONFIG_COND_NE:
       yymsp[0].minor.yy18->key.ptr[pos] = '='; /* opposite cond */
@@ -1307,20 +1503,20 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1310 "./configparser.c"
+#line 1506 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 32:
-#line 513 "../../src/configparser.y"
+#line 709 "../../src/configparser.y"
 {
   yygotominor.yy18 = yymsp[0].minor.yy18;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1320 "./configparser.c"
+#line 1516 "./configparser.c"
         break;
       case 33:
-#line 518 "../../src/configparser.y"
+#line 714 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1334,14 +1530,14 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1337 "./configparser.c"
+#line 1533 "./configparser.c"
         /* No destructor defined for context */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 34:
-#line 532 "../../src/configparser.y"
+#line 728 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1355,18 +1551,15 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1358 "./configparser.c"
+#line 1554 "./configparser.c"
         /* No destructor defined for context_else */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 35:
-#line 546 "../../src/configparser.y"
+#line 742 "../../src/configparser.y"
 {
-  data_config *dc;
-  buffer *b = NULL, *rvalue;
-  const char *op = NULL;
 
   if (ctx->ok && yymsp[0].minor.yy91->type != TYPE_STRING) {
     fprintf(stderr, "rvalue must be string");
@@ -1374,245 +1567,64 @@ static void yy_reduce(
   }
 
   if (ctx->ok) {
-    switch(yymsp[-1].minor.yy53) {
-    case CONFIG_COND_NE:
-      op = "!=";
-      break;
-    case CONFIG_COND_EQ:
-      op = "==";
-      break;
-    case CONFIG_COND_NOMATCH:
-      op = "!~";
-      break;
-    case CONFIG_COND_MATCH:
-      op = "=~";
-      break;
-    default:
-      force_assert(0);
-      return; /* unreachable */
-    }
-
-    b = buffer_init();
-    buffer_copy_buffer(b, &ctx->current->key);
-    buffer_append_string_len(b, CONST_STR_LEN("/"));
-    buffer_append_string_buffer(b, yymsp[-5].minor.yy0);
-    buffer_append_string_buffer(b, yymsp[-3].minor.yy29);
-    buffer_append_string_len(b, op, 2);
-    rvalue = &((data_string*)yymsp[0].minor.yy91)->value;
-    buffer_append_string_buffer(b, rvalue);
-
-    if (NULL != (dc = configparser_get_data_config(ctx->all_configs, CONST_BUF_LEN(b)))) {
-      configparser_push(ctx, dc, 0);
-    } else {
-      static const struct {
-        comp_key_t comp;
-        char *comp_key;
-        size_t len;
-      } comps[] = {
-        { COMP_SERVER_SOCKET,      CONST_STR_LEN("SERVER[\"socket\"]"   ) },
-        { COMP_HTTP_URL,           CONST_STR_LEN("HTTP[\"url\"]"        ) },
-        { COMP_HTTP_HOST,          CONST_STR_LEN("HTTP[\"host\"]"       ) },
-        { COMP_HTTP_REQUEST_HEADER,CONST_STR_LEN("HTTP[\"referer\"]"    ) },
-        { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
-        { COMP_HTTP_REQUEST_HEADER,CONST_STR_LEN("HTTP[\"user-agent\"]" ) },
-        { COMP_HTTP_LANGUAGE,      CONST_STR_LEN("HTTP[\"language\"]"   ) },
-        { COMP_HTTP_REQUEST_HEADER,CONST_STR_LEN("HTTP[\"cookie\"]"     ) },
-        { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
-        { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remote-ip\"]"   ) },
-        { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"querystring\"]") },
-        { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"query-string\"]") },
-        { COMP_HTTP_REQUEST_METHOD, CONST_STR_LEN("HTTP[\"request-method\"]") },
-        { COMP_HTTP_SCHEME,        CONST_STR_LEN("HTTP[\"scheme\"]"     ) },
-        { COMP_UNSET, NULL, 0 },
-      };
-      size_t i;
-
-      dc = data_config_init();
-
-      buffer_copy_buffer(&dc->key, b);
-      dc->op = op;
-      buffer_copy_buffer(dc->comp_tag, yymsp[-3].minor.yy29);
-      buffer_copy_buffer(dc->comp_key, yymsp[-5].minor.yy0);
-      buffer_append_string_len(dc->comp_key, CONST_STR_LEN("[\""));
-      buffer_append_string_buffer(dc->comp_key, yymsp[-3].minor.yy29);
-      buffer_append_string_len(dc->comp_key, CONST_STR_LEN("\"]"));
-      dc->cond = yymsp[-1].minor.yy53;
-
-      for (i = 0; comps[i].comp_key; i ++) {
-        if (buffer_is_equal_string(
-              dc->comp_key, comps[i].comp_key, comps[i].len)) {
-          dc->comp = comps[i].comp;
-          break;
-        }
-      }
-      if (COMP_UNSET == dc->comp) {
-        if (buffer_is_equal_string(yymsp[-5].minor.yy0, CONST_STR_LEN("REQUEST_HEADER"))) {
-          dc->comp = COMP_HTTP_REQUEST_HEADER;
-        }
-        else {
-          fprintf(stderr, "error comp_key %s", dc->comp_key->ptr);
-          ctx->ok = 0;
-        }
-      }
-      else if (COMP_HTTP_LANGUAGE == dc->comp) {
-        dc->comp = COMP_HTTP_REQUEST_HEADER;
-        buffer_copy_string_len(dc->comp_tag, CONST_STR_LEN("Accept-Language"));
-      }
-      else if (COMP_HTTP_USER_AGENT == dc->comp) {
-        dc->comp = COMP_HTTP_REQUEST_HEADER;
-        buffer_copy_string_len(dc->comp_tag, CONST_STR_LEN("User-Agent"));
-      }
-      else if (COMP_HTTP_REMOTE_IP == dc->comp
-               && (dc->cond == CONFIG_COND_EQ || dc->cond == CONFIG_COND_NE)) {
-        char * const slash = strchr(rvalue->ptr, '/'); /* CIDR mask */
-        char * const colon = strchr(rvalue->ptr, ':'); /* IPv6 */
-        if (NULL != slash && slash == rvalue->ptr){/*(skip AF_UNIX /path/file)*/
-        }
-        else if (NULL != slash) {
-          char *nptr;
-          const unsigned long nm_bits = strtoul(slash + 1, &nptr, 10);
-          if (*nptr || 0 == nm_bits || nm_bits > (NULL != colon ? 128 : 32)) {
-            /*(also rejects (slash+1 == nptr) which results in nm_bits = 0)*/
-            fprintf(stderr, "invalid or missing netmask: %s\n", rvalue->ptr);
-            ctx->ok = 0;
-          }
-          else {
-            int rc;
-            buffer_string_set_length(rvalue, (size_t)(slash - rvalue->ptr)); /*(truncate)*/
-            rc = (NULL == colon)
-              ? http_request_host_normalize(rvalue, 0)
-              : configparser_remoteip_normalize_compat(rvalue);
-            buffer_append_string_len(rvalue, CONST_STR_LEN("/"));
-            buffer_append_int(rvalue, (int)nm_bits);
-            if (0 != rc) {
-              fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
-              ctx->ok = 0;
-            }
-          }
-        }
-        else {
-          int rc = (NULL == colon)
-            ? http_request_host_normalize(rvalue, 0)
-            : configparser_remoteip_normalize_compat(rvalue);
-          if (0 != rc) {
-            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
-            ctx->ok = 0;
-          }
-        }
-      }
-      else if (COMP_SERVER_SOCKET == dc->comp) {
-        /*(redundant with parsing in network.c; not actually required here)*/
-        if (rvalue->ptr[0] != ':' /*(network.c special-cases ":" and "[]")*/
-            && !(rvalue->ptr[0] == '[' && rvalue->ptr[1] == ']')) {
-          if (http_request_host_normalize(rvalue, 0)) {
-            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
-            ctx->ok = 0;
-          }
-        }
-      }
-      else if (COMP_HTTP_HOST == dc->comp) {
-        if (dc->cond == CONFIG_COND_EQ || dc->cond == CONFIG_COND_NE) {
-          if (http_request_host_normalize(rvalue, 0)) {
-            fprintf(stderr, "invalid IP addr: %s\n", rvalue->ptr);
-            ctx->ok = 0;
-          }
-        }
-      }
-
-      if (COMP_HTTP_REQUEST_HEADER == dc->comp) {
-        dc->ext = http_header_hkey_get(CONST_BUF_LEN(dc->comp_tag));
-      }
-
-      buffer_copy_buffer(&dc->string, rvalue);
-
-      if (ctx->ok) switch(yymsp[-1].minor.yy53) {
-      case CONFIG_COND_NE:
-      case CONFIG_COND_EQ:
-        break;
-      case CONFIG_COND_NOMATCH:
-      case CONFIG_COND_MATCH: {
-        if (!data_config_pcre_compile(dc)) {
-          ctx->ok = 0;
-        }
-        break;
-      }
-
-      default:
-        fprintf(stderr, "unknown condition for $%s[%s]\n",
-                        yymsp[-5].minor.yy0->ptr, yymsp[-3].minor.yy29->ptr);
-        ctx->ok = 0;
-        break;
-      }
-
-      if (ctx->ok) {
-        configparser_push(ctx, dc, 1);
-      } else {
-        dc->fn->free((data_unset*) dc);
-      }
-    }
+    configparser_parse_condition(ctx, yymsp[-5].minor.yy0, yymsp[-3].minor.yy29, yymsp[-1].minor.yy53, &((data_string *)yymsp[0].minor.yy91)->value);
   }
 
-  buffer_free(b);
   buffer_free(yymsp[-5].minor.yy0);
   yymsp[-5].minor.yy0 = NULL;
   buffer_free(yymsp[-3].minor.yy29);
   yymsp[-3].minor.yy29 = NULL;
-  if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
+  yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1563 "./configparser.c"
+#line 1580 "./configparser.c"
   yy_destructor(17,&yymsp[-6].minor);
   yy_destructor(19,&yymsp[-4].minor);
   yy_destructor(20,&yymsp[-2].minor);
         break;
       case 36:
-#line 744 "../../src/configparser.y"
+#line 761 "../../src/configparser.y"
 {
   if (ctx->ok) {
-    data_config *dc = data_config_init();
-    buffer_copy_buffer(&dc->key, &ctx->current->key);
-    buffer_append_string_len(&dc->key, CONST_STR_LEN("/"));
-    buffer_append_string_len(&dc->key, CONST_STR_LEN("else_tmp_token"));
-    dc->cond = CONFIG_COND_ELSE;
-    configparser_push(ctx, dc, 1);
+    configparser_parse_else_condition(ctx);
   }
 }
-#line 1580 "./configparser.c"
+#line 1592 "./configparser.c"
         break;
       case 37:
-#line 755 "../../src/configparser.y"
+#line 767 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_EQ;
 }
-#line 1587 "./configparser.c"
+#line 1599 "./configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 38:
-#line 758 "../../src/configparser.y"
+#line 770 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_MATCH;
 }
-#line 1595 "./configparser.c"
+#line 1607 "./configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 39:
-#line 761 "../../src/configparser.y"
+#line 773 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NE;
 }
-#line 1603 "./configparser.c"
+#line 1615 "./configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 40:
-#line 764 "../../src/configparser.y"
+#line 776 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NOMATCH;
 }
-#line 1611 "./configparser.c"
+#line 1623 "./configparser.c"
   yy_destructor(24,&yymsp[0].minor);
         break;
       case 41:
-#line 768 "../../src/configparser.y"
+#line 780 "../../src/configparser.y"
 {
   yygotominor.yy29 = NULL;
   if (ctx->ok) {
@@ -1629,10 +1641,10 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->fn->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1632 "./configparser.c"
+#line 1644 "./configparser.c"
         break;
       case 42:
-#line 785 "../../src/configparser.y"
+#line 797 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1642,11 +1654,11 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1645 "./configparser.c"
+#line 1657 "./configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
       case 43:
-#line 795 "../../src/configparser.y"
+#line 807 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1656,7 +1668,7 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1659 "./configparser.c"
+#line 1671 "./configparser.c"
   yy_destructor(26,&yymsp[-1].minor);
         break;
   };
@@ -1686,11 +1698,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 155 "../../src/configparser.y"
+#line 350 "../../src/configparser.y"
 
   ctx->ok = 0;
 
-#line 1693 "./configparser.c"
+#line 1705 "./configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
