@@ -2,7 +2,9 @@
 #define _CONNECTIONS_H_
 #include "first.h"
 
-#include "base.h"
+#include "base_decls.h"
+
+struct server_socket;   /* declaration */
 
 __attribute_cold__
 void connections_free(server *srv);
@@ -14,8 +16,8 @@ void connection_periodic_maint (server *srv, time_t cur_ts);
 
 int connection_send_1xx (request_st *r, connection *con);
 
-connection * connection_accept(server *srv, server_socket *srv_sock);
-connection * connection_accepted(server *srv, server_socket *srv_socket, sock_addr *cnt_addr, int cnt);
+connection * connection_accept(server *srv, struct server_socket *srv_sock);
+connection * connection_accepted(server *srv, struct server_socket *srv_socket, sock_addr *cnt_addr, int cnt);
 
 void connection_state_machine(connection *con);
 
