@@ -6,12 +6,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+__attribute_cold__
 static data_unset *data_integer_copy(const data_unset *s) {
 	data_integer *src = (data_integer *)s;
 	data_integer *ds = data_integer_init();
 
-	buffer_copy_buffer(ds->key, src->key);
-	ds->is_index_key = src->is_index_key;
+	if (!buffer_is_empty(&src->key)) buffer_copy_buffer(&ds->key, &src->key);
 	ds->value = src->value;
 	return (data_unset *)ds;
 }
@@ -19,19 +19,12 @@ static data_unset *data_integer_copy(const data_unset *s) {
 static void data_integer_free(data_unset *d) {
 	data_integer *ds = (data_integer *)d;
 
-	buffer_free(ds->key);
+	free(ds->key.ptr);
 
 	free(d);
 }
 
-static void data_integer_reset(data_unset *d) {
-	data_integer *ds = (data_integer *)d;
-
-	/* reused integer elements */
-	buffer_clear(ds->key);
-	ds->value = 0;
-}
-
+__attribute_cold__
 static int data_integer_insert_dup(data_unset *dst, data_unset *src) {
 	UNUSED(dst);
 
@@ -40,6 +33,7 @@ static int data_integer_insert_dup(data_unset *dst, data_unset *src) {
 	return 0;
 }
 
+__attribute_cold__
 static void data_integer_print(const data_unset *d, int depth) {
 	data_integer *ds = (data_integer *)d;
 	UNUSED(depth);
@@ -50,7 +44,6 @@ static void data_integer_print(const data_unset *d, int depth) {
 
 data_integer *data_integer_init(void) {
 	static const struct data_methods fn = {
-		data_integer_reset,
 		data_integer_copy,
 		data_integer_free,
 		data_integer_insert_dup,
@@ -61,7 +54,6 @@ data_integer *data_integer_init(void) {
 	ds = calloc(1, sizeof(*ds));
 	force_assert(NULL != ds);
 
-	ds->key = buffer_init();
 	ds->value = 0;
 
 	ds->type = TYPE_INTEGER;
