@@ -69,8 +69,8 @@ typedef enum {
 typedef struct {
     int *ptr;
 
-    size_t used;
-    size_t size;
+    uint32_t used;
+    uint32_t size;
 } buffer_int;
 #endif
 
@@ -82,8 +82,9 @@ struct fdevents {
     int (*event_del)(struct fdevents *ev, fdnode *fdn);
     int (*poll)(struct fdevents *ev, int timeout_ms);
 
-    struct server *srv;
-    size_t maxfds;
+    log_error_st *errh;
+    int *cur_fds;
+    uint32_t maxfds;
   #ifdef FDEVENT_USE_LINUX_EPOLL
     int epoll_fd;
     struct epoll_event *epoll_events;
@@ -106,8 +107,8 @@ struct fdevents {
   #ifdef FDEVENT_USE_POLL
     struct pollfd *pollfds;
 
-    size_t size;
-    size_t used;
+    uint32_t size;
+    uint32_t used;
 
     buffer_int unused;
   #endif
@@ -125,6 +126,7 @@ struct fdevents {
 
     int (*reset)(struct fdevents *ev);
     void (*free)(struct fdevents *ev);
+    const char *event_handler;
     fdevent_handler_t type;
 };
 
