@@ -737,6 +737,10 @@ static void http_response_xsendfile (request_st * const r, buffer * const path,
 	if (r->conf.force_lowercase_filenames) {
 		buffer_to_lower(path);
 	}
+	if (buffer_string_is_empty(path)) {
+		r->http_status = 502;
+		valid = 0;
+	}
 
 	/* check that path is under xdocroot(s)
 	 * - xdocroot should have trailing slash appended at config time
@@ -815,6 +819,10 @@ static void http_response_xsendfile2(request_st * const r, const buffer * const
         if (r->conf.force_lowercase_filenames) {
             buffer_to_lower(b);
         }
+        if (buffer_string_is_empty(b)) {
+            r->http_status = 502;
+            break;
+        }
         if (xdocroot) {
             size_t i, xlen = buffer_string_length(b);
             for (i = 0; i < xdocroot->used; ++i) {
@@ -1746,8 +1754,8 @@ int http_cgi_headers (request_st * const r, http_cgi_opts * const opts, http_cgi
             /* Security: Do not emit HTTP_PROXY in environment.
              * Some executables use HTTP_PROXY to configure
              * outgoing proxy.  See also https://httpoxy.org/ */
-            if (buffer_is_equal_caseless_string(&ds->key,
-                                                CONST_STR_LEN("Proxy"))) {
+            if (ds->ext == HTTP_HEADER_OTHER
+                && buffer_eq_icase_slen(&ds->key, CONST_STR_LEN("Proxy"))) {
                 continue;
             }
             buffer_copy_string_encoded_cgi_varnames(tb,
