@@ -10,6 +10,8 @@
 
 #include <lauxlib.h>
 
+#include "sys-crypto-md.h"
+
 #include "mod_cml_funcs.h"
 #include "mod_cml.h"
 
@@ -17,8 +19,6 @@
 #include "log.h"
 #include "plugin.h"
 
-#include "md5.h"
-
 #define HASHLEN 16
 typedef unsigned char HASH[HASHLEN];
 #define HASHHEXLEN 32
