@@ -101,7 +101,7 @@ static int cache_export_get_params(lua_State *L, int tbl, buffer *qrystr) {
 	return 0;
 }
 
-int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
+int cache_parse_lua(request_st * const r, plugin_data * const p, const buffer * const fn) {
 	lua_State *L;
 	int ret = -1;
 	buffer *b;
@@ -137,22 +137,22 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	{
 		int header_tbl = lua_gettop(L);
 
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("REQUEST_URI"), CONST_BUF_LEN(con->request.orig_uri));
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("SCRIPT_NAME"), CONST_BUF_LEN(con->uri.path));
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("SCRIPT_FILENAME"), CONST_BUF_LEN(con->physical.path));
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("DOCUMENT_ROOT"), CONST_BUF_LEN(con->physical.basedir));
-		if (!buffer_string_is_empty(con->request.pathinfo)) {
-			c_to_lua_push(L, header_tbl, CONST_STR_LEN("PATH_INFO"), CONST_BUF_LEN(con->request.pathinfo));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("REQUEST_URI"), CONST_BUF_LEN(&r->target_orig));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("SCRIPT_NAME"), CONST_BUF_LEN(&r->uri.path));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("SCRIPT_FILENAME"), CONST_BUF_LEN(&r->physical.path));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("DOCUMENT_ROOT"), CONST_BUF_LEN(&r->physical.basedir));
+		if (!buffer_string_is_empty(&r->pathinfo)) {
+			c_to_lua_push(L, header_tbl, CONST_STR_LEN("PATH_INFO"), CONST_BUF_LEN(&r->pathinfo));
 		}
 
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("CWD"), CONST_BUF_LEN(p->basedir));
-		c_to_lua_push(L, header_tbl, CONST_STR_LEN("BASEURL"), CONST_BUF_LEN(p->baseurl));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("CWD"), CONST_BUF_LEN(&p->basedir));
+		c_to_lua_push(L, header_tbl, CONST_STR_LEN("BASEURL"), CONST_BUF_LEN(&p->baseurl));
 	}
 	lua_setglobal(L, "request");
 
 	/* register GET parameter */
 	lua_newtable(L);
-	cache_export_get_params(L, lua_gettop(L), con->uri.query);
+	cache_export_get_params(L, lua_gettop(L), &r->uri.query);
 	lua_setglobal(L, "get");
 
 	/* 2 default constants */
@@ -165,20 +165,16 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	/* load lua program */
 	ret = luaL_loadfile(L, fn->ptr);
 	if (0 != ret) {
-		log_error_write(srv, __FILE__, __LINE__, "sbsS",
-			"failed loading cml_lua script",
-			fn,
-			":",
-			lua_tostring(L, -1));
+		log_error(r->conf.errh, __FILE__, __LINE__,
+		  "failed loading cml_lua script %s: %s",
+		  fn->ptr, lua_tostring(L, -1));
 		goto error;
 	}
 
 	if (lua_pcall(L, 0, 1, 0)) {
-		log_error_write(srv, __FILE__, __LINE__, "sbsS",
-			"failed running cml_lua script",
-			fn,
-			":",
-			lua_tostring(L, -1));
+		log_error(r->conf.errh, __FILE__, __LINE__,
+		  "failed running cml_lua script %s: %s",
+		  fn->ptr, lua_tostring(L, -1));
 		goto error;
 	}
 
@@ -187,10 +183,10 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 	lua_pop(L, 1);
 
 	/* fetch the data from lua */
-	lua_to_c_get_string(L, "trigger_handler", p->trigger_handler);
+	lua_to_c_get_string(L, "trigger_handler", &p->trigger_handler);
 
 	if (0 == lua_to_c_get_string(L, "output_contenttype", b)) {
-		http_header_response_set(con, HTTP_HEADER_CONTENT_TYPE, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(b));
+		http_header_response_set(r, HTTP_HEADER_CONTENT_TYPE, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(b));
 	}
 
 	if (ret == 0) {
@@ -200,8 +196,8 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 		time_t mtime = 0;
 
 		if (!lua_to_c_is_table(L, "output_include")) {
-			log_error_write(srv, __FILE__, __LINE__, "s",
-				"output_include is missing or not a table");
+			log_error(r->conf.errh, __FILE__, __LINE__,
+			  "output_include is missing or not a table");
 			ret = -1;
 
 			goto error;
@@ -224,38 +220,39 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 			/* key' is at index -2 and value' at index -1 */
 
 			if (lua_isstring(L, -1)) {
-				const char *s = lua_tostring(L, -1);
+				size_t slen;
+				const char * const s = lua_tolstring(L, -1, &slen);
 				struct stat st;
 				int fd;
 
 				/* the file is relative, make it absolute */
 				if (s[0] != '/') {
-					buffer_copy_buffer(b, p->basedir);
-					buffer_append_string(b, lua_tostring(L, -1));
+					buffer_copy_buffer(b, &p->basedir);
+					buffer_append_path_len(b, s, (uint32_t)slen);
 				} else {
-					buffer_copy_string(b, lua_tostring(L, -1));
+					buffer_copy_string_len(b, s, (uint32_t)slen);
 				}
 
-				fd = stat_cache_open_rdonly_fstat(b, &st, con->conf.follow_symlink);
+				fd = stat_cache_open_rdonly_fstat(b, &st, r->conf.follow_symlink);
 				if (fd < 0) {
 					/* stat failed */
 
 					switch(errno) {
 					case ENOENT:
 						/* a file is missing, call the handler to generate it */
-						if (!buffer_string_is_empty(p->trigger_handler)) {
+						if (!buffer_string_is_empty(&p->trigger_handler)) {
 							ret = 1; /* cache-miss */
 
-							log_error_write(srv, __FILE__, __LINE__, "s",
-									"a file is missing, calling handler");
+							log_error(r->conf.errh, __FILE__, __LINE__,
+							  "a file is missing, calling handler");
 
 							break;
 						} else {
 							/* handler not set -> 500 */
 							ret = -1;
 
-							log_error_write(srv, __FILE__, __LINE__, "s",
-									"a file missing and no handler set");
+							log_error(r->conf.errh, __FILE__, __LINE__,
+							  "a file missing and no handler set");
 
 							break;
 						}
@@ -264,14 +261,13 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 						break;
 					}
 				} else {
-					chunkqueue_append_file_fd(con->write_queue, b, fd, 0, st.st_size);
+					chunkqueue_append_file_fd(&r->write_queue, b, fd, 0, st.st_size);
 					if (st.st_mtime > mtime) mtime = st.st_mtime;
 				}
 			} else {
 				/* not a string */
 				ret = -1;
-				log_error_write(srv, __FILE__, __LINE__, "s",
-						"not a string");
+				log_error(r->conf.errh, __FILE__, __LINE__, "not a string");
 				break;
 			}
 
@@ -281,39 +277,35 @@ int cache_parse_lua(server *srv, connection *con, plugin_data *p, buffer *fn) {
 		lua_settop(L, curelem - 1);
 
 		if (ret == 0) {
-			buffer *vb = http_header_response_get(con, HTTP_HEADER_LAST_MODIFIED, CONST_STR_LEN("Last-Modified"));
+			const buffer *vb = http_header_response_get(r, HTTP_HEADER_LAST_MODIFIED, CONST_STR_LEN("Last-Modified"));
 			if (NULL == vb) { /* no Last-Modified specified */
-				char timebuf[sizeof("Sat, 23 Jul 2005 21:20:01 GMT")];
 				if (0 == mtime) mtime = time(NULL); /* default last-modified to now */
-				strftime(timebuf, sizeof(timebuf), "%a, %d %b %Y %H:%M:%S GMT", gmtime(&mtime));
-				http_header_response_set(con, HTTP_HEADER_LAST_MODIFIED, CONST_STR_LEN("Last-Modified"), timebuf, sizeof(timebuf) - 1);
-				vb = http_header_response_get(con, HTTP_HEADER_LAST_MODIFIED, CONST_STR_LEN("Last-Modified"));
-				force_assert(NULL != vb);
+				vb = http_response_set_last_modified(r, mtime);
 			}
 
-			con->file_finished = 1;
+			r->resp_body_finished = 1;
 
-			if (HANDLER_FINISHED == http_response_handle_cachable(srv, con, vb)) {
+			if (HANDLER_FINISHED == http_response_handle_cachable(r, vb, mtime)) {
 				/* ok, the client already has our content,
 				 * no need to send it again */
 
-				chunkqueue_reset(con->write_queue);
+				chunkqueue_reset(&r->write_queue);
 				ret = 0; /* cache-hit */
 			}
 		} else {
-			chunkqueue_reset(con->write_queue);
+			chunkqueue_reset(&r->write_queue);
 		}
 	}
 
-	if (ret == 1 && !buffer_string_is_empty(p->trigger_handler)) {
+	if (ret == 1 && !buffer_string_is_empty(&p->trigger_handler)) {
 		/* cache-miss */
-		buffer_copy_buffer(con->uri.path, p->baseurl);
-		buffer_append_string_buffer(con->uri.path, p->trigger_handler);
+		buffer_copy_buffer(&r->uri.path, &p->baseurl);
+		buffer_append_string_buffer(&r->uri.path, &p->trigger_handler);
 
-		buffer_copy_buffer(con->physical.path, p->basedir);
-		buffer_append_string_buffer(con->physical.path, p->trigger_handler);
+		buffer_copy_buffer(&r->physical.path, &p->basedir);
+		buffer_append_path_len(&r->physical.path, CONST_BUF_LEN(&p->trigger_handler));
 
-		chunkqueue_reset(con->write_queue);
+		chunkqueue_reset(&r->write_queue);
 	}
 
 error:
