@@ -144,6 +144,11 @@
 # include <brotli/encode.h>
 #endif
 
+#if defined HAVE_ZSTD_H && defined HAVE_ZSTD
+# define USE_ZSTD
+# include <zstd.h>
+#endif
+
 #if defined HAVE_SYS_MMAN_H && defined HAVE_MMAP && defined ENABLE_MMAP
 #define USE_MMAP
 
@@ -170,6 +175,7 @@ static void sigbus_handler(int sig) {
 #define HTTP_ACCEPT_ENCODING_X_GZIP   BV(5)
 #define HTTP_ACCEPT_ENCODING_X_BZIP2  BV(6)
 #define HTTP_ACCEPT_ENCODING_BR       BV(7)
+#define HTTP_ACCEPT_ENCODING_ZSTD     BV(8)
 
 typedef struct {
 	const array	*mimetypes;
@@ -202,6 +208,9 @@ typedef struct {
 	      #endif
 	      #ifdef USE_BROTLI
 		BrotliEncoderState *br;
+	      #endif
+	      #ifdef USE_ZSTD
+		ZSTD_CStream *cctx;
 	      #endif
 		int dummy;
 	} u;
@@ -244,7 +253,11 @@ static void handler_ctx_free(handler_ctx *hctx) {
 
 INIT_FUNC(mod_deflate_init) {
     plugin_data * const p = calloc(1, sizeof(plugin_data));
+  #ifdef USE_ZSTD
+    buffer_string_prepare_copy(&p->tmp_buf, ZSTD_CStreamOutSize());
+  #else
     buffer_string_prepare_copy(&p->tmp_buf, 65536);
+  #endif
     return p;
 }
 
@@ -384,7 +397,8 @@ static short mod_deflate_encodings_to_flags(const array *encodings) {
     short allowed_encodings = 0;
     if (encodings->used) {
         for (uint32_t j = 0; j < encodings->used; ++j) {
-          #if defined(USE_ZLIB) || defined(USE_BZ2LIB) || defined(USE_BROTLI)
+          #if defined(USE_ZLIB) || defined(USE_BZ2LIB) || defined(USE_BROTLI) \
+           || defined(USE_ZSTD)
             data_string *ds = (data_string *)encodings->data[j];
           #endif
           #ifdef USE_ZLIB
@@ -411,6 +425,10 @@ static short mod_deflate_encodings_to_flags(const array *encodings) {
             if (NULL != strstr(ds->value.ptr, "br"))
                 allowed_encodings |= HTTP_ACCEPT_ENCODING_BR;
           #endif
+          #ifdef USE_ZSTD
+            if (NULL != strstr(ds->value.ptr, "zstd"))
+                allowed_encodings |= HTTP_ACCEPT_ENCODING_ZSTD;
+          #endif
         }
     }
     else {
@@ -427,6 +445,9 @@ static short mod_deflate_encodings_to_flags(const array *encodings) {
       #ifdef USE_BROTLI
         allowed_encodings |= HTTP_ACCEPT_ENCODING_BR;
       #endif
+      #ifdef USE_ZSTD
+        allowed_encodings |= HTTP_ACCEPT_ENCODING_ZSTD;
+      #endif
     }
     return allowed_encodings;
 }
@@ -612,7 +633,8 @@ SETDEFAULTS_FUNC(mod_deflate_set_defaults) {
 }
 
 
-#if defined(USE_ZLIB) || defined(USE_BZ2LIB) || defined(USE_BROTLI)
+#if defined(USE_ZLIB) || defined(USE_BZ2LIB) || defined(USE_BROTLI) \
+ || defined(USE_ZSTD)
 static int mod_deflate_cache_file_append (handler_ctx * const hctx, const char *out, size_t len) {
     ssize_t wr;
     do {
@@ -933,6 +955,89 @@ static int stream_br_end(handler_ctx *hctx) {
 #endif
 
 
+#ifdef USE_ZSTD
+
+static int stream_zstd_init(handler_ctx *hctx) {
+    ZSTD_CStream * const cctx = hctx->u.cctx = ZSTD_createCStream();
+    if (NULL == cctx) return -1;
+    hctx->output->used = 0;
+
+    /* future: consider allowing tunables by encoder algorithm,
+     * (i.e. not generic "compression_level" across all compression algos) */
+    /*(note: we ignore any errors while tuning parameters here)*/
+    const plugin_data * const p = hctx->plugin_data;
+    if (p->conf.compression_level >= 0) { /* -1 is lighttpd default for "unset" */
+        int level = p->conf.compression_level;
+      #if ZSTD_VERSION_NUMBER >= 10000+400+0 /* v1.4.0 */
+        ZSTD_CCtx_setParameter(cctx, ZSTD_c_compressionLevel, level);
+      #else
+        ZSTD_initCStream(cctx, level);
+      #endif
+    }
+    return 0;
+}
+
+static int stream_zstd_compress(handler_ctx * const hctx, unsigned char * const start, off_t st_size) {
+    ZSTD_CStream * const cctx = hctx->u.cctx;
+    ZSTD_inBuffer zib = { start, (size_t)st_size, 0 };
+    ZSTD_outBuffer zob = { hctx->output->ptr,
+                           hctx->output->size,
+                           hctx->output->used };
+    hctx->output->used = 0;
+    hctx->bytes_in += st_size;
+    while (zib.pos < zib.size) {
+      #if ZSTD_VERSION_NUMBER >= 10000+400+0 /* v1.4.0 */
+        const size_t rv = ZSTD_compressStream2(cctx,&zob,&zib,ZSTD_e_continue);
+      #else
+        const size_t rv = ZSTD_compressStream(cctx, &zob, &zib);
+      #endif
+        if (ZSTD_isError(rv)) return -1;
+        if (zib.pos == zib.size) break; /* defer flush */
+        hctx->bytes_out += (off_t)zob.pos;
+        if (0 != stream_http_chunk_append_mem(hctx, zob.dst, zob.pos))
+            return -1;
+        zob.pos = 0;
+    }
+    hctx->output->used = (uint32_t)zob.pos;
+    return 0;
+}
+
+static int stream_zstd_flush(handler_ctx * const hctx, int end) {
+    ZSTD_CStream * const cctx = hctx->u.cctx;
+  #if ZSTD_VERSION_NUMBER >= 10000+400+0 /* v1.4.0 */
+    const ZSTD_EndDirective endOp = end ? ZSTD_e_end : ZSTD_e_flush;
+    ZSTD_inBuffer zib = { NULL, 0, 0 };
+  #endif
+    ZSTD_outBuffer zob = { hctx->output->ptr,
+                           hctx->output->size,
+                           hctx->output->used };
+    size_t rv;
+    do {
+      #if ZSTD_VERSION_NUMBER >= 10000+400+0 /* v1.4.0 */
+        rv = ZSTD_compressStream2(cctx, &zob, &zib, endOp);
+      #else
+        rv = end
+           ? ZSTD_endStream(cctx, &zob)
+           : ZSTD_flushStream(cctx, &zob);
+      #endif
+        if (ZSTD_isError(rv)) return -1;
+        hctx->bytes_out += (off_t)zob.pos;
+        if (0 != stream_http_chunk_append_mem(hctx, zob.dst, zob.pos))
+            return -1;
+        zob.pos = 0;
+    } while (0 != rv);
+    return 0;
+}
+
+static int stream_zstd_end(handler_ctx *hctx) {
+    ZSTD_CStream * const cctx = hctx->u.cctx;
+    ZSTD_freeCStream(cctx);
+    return 0;
+}
+
+#endif
+
+
 static int mod_deflate_stream_init(handler_ctx *hctx) {
 	switch(hctx->compression_type) {
 #ifdef USE_ZLIB
@@ -947,6 +1052,10 @@ static int mod_deflate_stream_init(handler_ctx *hctx) {
 #ifdef USE_BROTLI
 	case HTTP_ACCEPT_ENCODING_BR:
 		return stream_br_init(hctx);
+#endif
+#ifdef USE_ZSTD
+	case HTTP_ACCEPT_ENCODING_ZSTD:
+		return stream_zstd_init(hctx);
 #endif
 	default:
 		return -1;
@@ -968,6 +1077,10 @@ static int mod_deflate_compress(handler_ctx * const hctx, unsigned char * const
 #ifdef USE_BROTLI
 	case HTTP_ACCEPT_ENCODING_BR:
 		return stream_br_compress(hctx, start, st_size);
+#endif
+#ifdef USE_ZSTD
+	case HTTP_ACCEPT_ENCODING_ZSTD:
+		return stream_zstd_compress(hctx, start, st_size);
 #endif
 	default:
 		UNUSED(start);
@@ -990,6 +1103,10 @@ static int mod_deflate_stream_flush(handler_ctx * const hctx, int end) {
 #ifdef USE_BROTLI
 	case HTTP_ACCEPT_ENCODING_BR:
 		return stream_br_flush(hctx, end);
+#endif
+#ifdef USE_ZSTD
+	case HTTP_ACCEPT_ENCODING_ZSTD:
+		return stream_zstd_flush(hctx, end);
 #endif
 	default:
 		UNUSED(end);
@@ -1023,6 +1140,10 @@ static int mod_deflate_stream_end(handler_ctx *hctx) {
 #ifdef USE_BROTLI
 	case HTTP_ACCEPT_ENCODING_BR:
 		return stream_br_end(hctx);
+#endif
+#ifdef USE_ZSTD
+	case HTTP_ACCEPT_ENCODING_ZSTD:
+		return stream_zstd_end(hctx);
 #endif
 	default:
 		return -1;
@@ -1263,7 +1384,8 @@ static handler_t deflate_compress_response(request_st * const r, handler_ctx * c
 static int mod_deflate_choose_encoding (const char *value, plugin_data *p, const char **label) {
 	/* get client side support encodings */
 	int accept_encoding = 0;
-      #if !defined(USE_ZLIB) && !defined(USE_BZ2LIB) && !defined(USE_BROTLI)
+      #if !defined(USE_ZLIB) && !defined(USE_BZ2LIB) && !defined(USE_BROTLI) \
+       && !defined(USE_ZSTD)
 	UNUSED(value);
 	UNUSED(label);
       #else
@@ -1284,6 +1406,13 @@ static int mod_deflate_choose_encoding (const char *value, plugin_data *p, const
                #ifdef USE_ZLIB
                 if (0 == memcmp(v, "gzip", 4))
                     accept_encoding |= HTTP_ACCEPT_ENCODING_GZIP;
+               #endif
+               #ifdef USE_ZSTD
+                #ifdef USE_ZLIB
+                else
+                #endif
+                if (0 == memcmp(v, "zstd", 4))
+                    accept_encoding |= HTTP_ACCEPT_ENCODING_ZSTD;
                #endif
                 break;
               case 5:
@@ -1330,6 +1459,12 @@ static int mod_deflate_choose_encoding (const char *value, plugin_data *p, const
 	accept_encoding &= p->conf.allowed_encodings;
 
 	/* select best matching encoding */
+#ifdef USE_ZSTD
+	if (accept_encoding & HTTP_ACCEPT_ENCODING_ZSTD) {
+		*label = "zstd";
+		return HTTP_ACCEPT_ENCODING_ZSTD;
+	} else
+#endif
 #ifdef USE_BROTLI
 	if (accept_encoding & HTTP_ACCEPT_ENCODING_BR) {
 		*label = "br";
