@@ -405,6 +405,10 @@ SETDEFAULTS_FUNC(mod_extforward_set_defaults) {
         p->defaults.headers = p->default_headers = array_init(2);
         array_insert_value(p->default_headers,CONST_STR_LEN("X-Forwarded-For"));
         array_insert_value(p->default_headers,CONST_STR_LEN("Forwarded-For"));
+        for (uint32_t i = 0; i < p->default_headers->used; ++i) {
+            data_string * const ds = (data_string *)p->default_headers->data[i];
+            ds->ext = http_header_hkey_get(CONST_BUF_LEN(&ds->value));
+        }
     }
 
     /* attempt to warn if mod_extforward is not last module loaded to hook
@@ -425,6 +429,7 @@ SETDEFAULTS_FUNC(mod_extforward_set_defaults) {
             data_string *ds = (data_string *)srv->srvconf.modules->data[i];
             if (buffer_eq_slen(&ds->value, CONST_STR_LEN("mod_openssl"))
                 || buffer_eq_slen(&ds->value, CONST_STR_LEN("mod_mbedtls"))
+                || buffer_eq_slen(&ds->value, CONST_STR_LEN("mod_wolfssl"))
                 || buffer_eq_slen(&ds->value, CONST_STR_LEN("mod_nss"))
                 || buffer_eq_slen(&ds->value, CONST_STR_LEN("mod_gnutls"))) {
                 log_error(srv->errh, __FILE__, __LINE__,
@@ -598,7 +603,7 @@ static int mod_extforward_set_addr(request_st * const r, plugin_data *p, const c
 }
 
 static void mod_extforward_set_proto(request_st * const r, const char * const proto, size_t protolen) {
-	if (0 != protolen && !buffer_is_equal_caseless_string(&r->uri.scheme, proto, protolen)) {
+	if (0 != protolen && !buffer_eq_icase_slen(&r->uri.scheme, proto, protolen)) {
 		/* update scheme if X-Forwarded-Proto is set
 		 * Limitations:
 		 * - Only "http" or "https" are currently accepted since the request to lighttpd currently has to
@@ -1105,7 +1110,7 @@ URIHANDLER_FUNC(mod_extforward_uri_handler) {
 		const buffer * const hdr = &ds->value;
 		forwarded = http_header_request_get(r, ds->ext, CONST_BUF_LEN(hdr));
 		if (forwarded) {
-			is_forwarded_header = buffer_is_equal_caseless_string(hdr, CONST_STR_LEN("Forwarded"));
+			is_forwarded_header = (ds->ext == HTTP_HEADER_FORWARDED);
 			break;
 		}
 	}
