@@ -1,3 +1,9 @@
+/*
+ * mod_vhostdb_dbi - virtual hosts mapping from backend DBI interface
+ *
+ * Copyright(c) 2017 Glenn Strauss gstrauss()gluelogic.com  All rights reserved
+ * License: BSD 3-clause (same as lighttpd)
+ */
 #include "first.h"
 
 #include <dbi/dbi.h>
@@ -24,19 +30,18 @@
 typedef struct {
     dbi_conn dbconn;
     dbi_inst dbinst;
-    buffer *sqlquery;
-    server *srv;
+    const buffer *sqlquery;
+    log_error_st *errh;
     short reconnect_count;
 } vhostdb_config;
 
 typedef struct {
     void *vdata;
-    array *options;
 } plugin_config;
 
 typedef struct {
     PLUGIN_DATA;
-    plugin_config **config_storage;
+    plugin_config defaults;
     plugin_config conf;
 } plugin_data;
 
@@ -55,8 +60,7 @@ static void mod_vhostdb_dbi_error_callback (dbi_conn dbconn, void *vdata)
     }
 
     dbi_conn_error(dbconn, &errormsg);
-    log_error_write(dbconf->srv, __FILE__, __LINE__, "ss",
-                    "dbi_conn_connect():", errormsg);
+    log_error(dbconf->errh,__FILE__,__LINE__,"dbi_conn_connect(): %s",errormsg);
 }
 
 static void mod_vhostdb_dbconf_free (void *vdata)
@@ -68,20 +72,20 @@ static void mod_vhostdb_dbconf_free (void *vdata)
     free(dbconf);
 }
 
-static int mod_vhostdb_dbconf_setup (server *srv, array *opts, void **vdata)
+static int mod_vhostdb_dbconf_setup (server *srv, const array *opts, void **vdata)
 {
-    buffer *sqlquery = NULL;
+    const buffer *sqlquery = NULL;
     const buffer *dbtype=NULL, *dbname=NULL;
 
     for (size_t i = 0; i < opts->used; ++i) {
         const data_string *ds = (data_string *)opts->data[i];
         if (ds->type == TYPE_STRING) {
-            if (buffer_is_equal_caseless_string(ds->key, CONST_STR_LEN("sql"))) {
-                sqlquery = ds->value;
-            } else if (buffer_is_equal_caseless_string(ds->key, CONST_STR_LEN("dbname"))) {
-                dbname = ds->value;
-            } else if (buffer_is_equal_caseless_string(ds->key, CONST_STR_LEN("dbtype"))) {
-                dbtype = ds->value;
+            if (buffer_is_equal_caseless_string(&ds->key, CONST_STR_LEN("sql"))) {
+                sqlquery = &ds->value;
+            } else if (buffer_is_equal_caseless_string(&ds->key, CONST_STR_LEN("dbname"))) {
+                dbname = &ds->value;
+            } else if (buffer_is_equal_caseless_string(&ds->key, CONST_STR_LEN("dbtype"))) {
+                dbtype = &ds->value;
             }
         }
     }
@@ -107,16 +111,16 @@ static int mod_vhostdb_dbconf_setup (server *srv, array *opts, void **vdata)
         dbi_inst dbinst = NULL;
         dbi_conn dbconn;
         if (dbi_initialize_r(NULL, &dbinst) < 1) {
-            log_error_write(srv, __FILE__, __LINE__, "s",
-                            "dbi_initialize_r() failed.  "
-                            "Do you have the DBD for this db type installed?");
+            log_error(srv->errh, __FILE__, __LINE__,
+              "dbi_initialize_r() failed.  "
+              "Do you have the DBD for this db type installed?");
             return -1;
         }
         dbconn = dbi_conn_new_r(dbtype->ptr, dbinst);
         if (NULL == dbconn) {
-            log_error_write(srv, __FILE__, __LINE__, "s",
-                            "dbi_conn_new_r() failed.  "
-                            "Do you have the DBD for this db type installed?");
+            log_error(srv->errh, __FILE__, __LINE__,
+              "dbi_conn_new_r() failed.  "
+              "Do you have the DBD for this db type installed?");
             dbi_shutdown_r(dbinst);
             return -1;
         }
@@ -124,15 +128,15 @@ static int mod_vhostdb_dbconf_setup (server *srv, array *opts, void **vdata)
         /* set options */
         for (size_t j = 0; j < opts->used; ++j) {
             data_unset *du = opts->data[j];
-            const buffer *opt = du->key;
+            const buffer *opt = &du->key;
             if (!buffer_string_is_empty(opt)) {
                 if (du->type == TYPE_INTEGER) {
                     data_integer *di = (data_integer *)du;
                     dbi_conn_set_option_numeric(dbconn, opt->ptr, di->value);
                 } else if (du->type == TYPE_STRING) {
                     data_string *ds = (data_string *)du;
-                    if (ds->value != sqlquery && ds->value != dbtype) {
-                        dbi_conn_set_option(dbconn, opt->ptr, ds->value->ptr);
+                    if (&ds->value != sqlquery && &ds->value != dbtype) {
+                        dbi_conn_set_option(dbconn, opt->ptr, ds->value.ptr);
                     }
                 }
             }
@@ -142,7 +146,7 @@ static int mod_vhostdb_dbconf_setup (server *srv, array *opts, void **vdata)
         dbconf->dbinst = dbinst;
         dbconf->dbconn = dbconn;
         dbconf->sqlquery = sqlquery;
-        dbconf->srv = srv;
+        dbconf->errh = srv->errh;
         dbconf->reconnect_count = 0;
         *vdata = dbconf;
 
@@ -151,15 +155,18 @@ static int mod_vhostdb_dbconf_setup (server *srv, array *opts, void **vdata)
 
         /* connect to database */
         mod_vhostdb_dbi_error_callback(dbconn, dbconf);
-        if (dbconf->reconnect_count >= 3) return -1;
+        if (dbconf->reconnect_count >= 3) {
+            mod_vhostdb_dbconf_free(dbconf);
+            return -1;
+        }
     }
 
     return 0;
 }
 
-static void mod_vhostdb_patch_connection (server *srv, connection *con, plugin_data *p);
+static void mod_vhostdb_patch_config (request_st * const r, plugin_data * const p);
 
-static int mod_vhostdb_dbi_query(server *srv, connection *con, void *p_d, buffer *docroot)
+static int mod_vhostdb_dbi_query(request_st * const r, void *p_d, buffer *docroot)
 {
     plugin_data *p = (plugin_data *)p_d;
     vhostdb_config *dbconf;
@@ -171,7 +178,7 @@ static int mod_vhostdb_dbi_query(server *srv, connection *con, void *p_d, buffer
     buffer *sqlquery = docroot;
     buffer_clear(sqlquery); /*(also resets docroot (alias))*/
 
-    mod_vhostdb_patch_connection(srv, con, p);
+    mod_vhostdb_patch_config(r, p);
     if (NULL == p->conf.vdata) return 0; /*(after resetting docroot)*/
     dbconf = (vhostdb_config *)p->conf.vdata;
 
@@ -179,7 +186,7 @@ static int mod_vhostdb_dbi_query(server *srv, connection *con, void *p_d, buffer
         if (NULL != (d = strchr(b, '?'))) {
             /* escape the uri.authority */
             char *esc = NULL;
-            size_t len = dbi_conn_escape_string_copy(dbconf->dbconn, con->uri.authority->ptr, &esc);
+            size_t len = dbi_conn_escape_string_copy(dbconf->dbconn, r->uri.authority.ptr, &esc);
             buffer_append_string_len(sqlquery, b, (size_t)(d - b));
             buffer_append_string_len(sqlquery, esc, len);
             free(esc);
@@ -203,7 +210,7 @@ static int mod_vhostdb_dbi_query(server *srv, connection *con, void *p_d, buffer
     if (!result) {
         const char *errmsg;
         dbi_conn_error(dbconf->dbconn, &errmsg);
-        log_error_write(srv, __FILE__, __LINE__, "s", errmsg);
+        log_error(r->conf.errh, __FILE__, __LINE__, "%s", errmsg);
         return -1;
     }
 
@@ -232,96 +239,100 @@ INIT_FUNC(mod_vhostdb_init) {
 }
 
 FREE_FUNC(mod_vhostdb_cleanup) {
-    plugin_data *p = p_d;
-    if (!p) return HANDLER_GO_ON;
-
-    if (p->config_storage) {
-        for (size_t i = 0; i < srv->config_context->used; i++) {
-            plugin_config *s = p->config_storage[i];
-            if (!s) continue;
-            mod_vhostdb_dbconf_free(s->vdata);
-            array_free(s->options);
-            free(s);
-        }
-        free(p->config_storage);
-    }
-    free(p);
-
-    UNUSED(srv);
-    return HANDLER_GO_ON;
-}
-
-SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
-    plugin_data *p = p_d;
-
-    config_values_t cv[] = {
-        { "vhostdb.dbi",    NULL, T_CONFIG_ARRAY,  T_CONFIG_SCOPE_CONNECTION },
-        { NULL,             NULL, T_CONFIG_UNSET,  T_CONFIG_SCOPE_UNSET }
-    };
-
-    p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
-
-    for (size_t i = 0; i < srv->config_context->used; ++i) {
-        data_config const *config = (data_config const*)srv->config_context->data[i];
-        plugin_config *s = calloc(1, sizeof(plugin_config));
-
-        s->options = array_init();
-        cv[0].destination = s->options;
-
-        p->config_storage[i] = s;
-
-        if (config_insert_values_global(srv, config->value, cv, i == 0 ? T_CONFIG_SCOPE_SERVER : T_CONFIG_SCOPE_CONNECTION)) {
-            return HANDLER_ERROR;
-        }
-
-	if (!array_is_kvany(s->options)) {
-		log_error_write(srv, __FILE__, __LINE__, "s",
-				"unexpected value for vhostdb.dbi; expected list of \"option\" => \"value\"");
-		return HANDLER_ERROR;
-	}
-
-	if (s->options->used
-            && 0 != mod_vhostdb_dbconf_setup(srv, s->options, &s->vdata)) {
-            return HANDLER_ERROR;
-        }
-    }
-
-    return HANDLER_GO_ON;
-}
-
-#define PATCH(x) \
-    p->conf.x = s->x;
-static void mod_vhostdb_patch_connection (server *srv, connection *con, plugin_data *p)
-{
-    plugin_config *s = p->config_storage[0];
-    PATCH(vdata);
-
-    /* skip the first, the global context */
-    for (size_t i = 1; i < srv->config_context->used; ++i) {
-        data_config *dc = (data_config *)srv->config_context->data[i];
-        s = p->config_storage[i];
-
-        /* condition didn't match */
-        if (!config_check_cond(srv, con, dc)) continue;
-
-        /* merge config */
-        for (size_t j = 0; j < dc->value->used; ++j) {
-            data_unset *du = dc->value->data[j];
-
-            if (buffer_is_equal_string(du->key, CONST_STR_LEN("vhostdb.dbi"))) {
-                PATCH(vdata);
+    plugin_data * const p = p_d;
+    if (NULL == p->cvlist) return;
+    /* (init i to 0 if global context; to 1 to skip empty global context) */
+    for (int i = !p->cvlist[0].v.u2[1], used = p->nconfig; i < used; ++i) {
+        config_plugin_value_t *cpv = p->cvlist + p->cvlist[i].v.u2[0];
+        for (; -1 != cpv->k_id; ++cpv) {
+            if (cpv->vtype != T_CONFIG_LOCAL || NULL == cpv->v.v) continue;
+            switch (cpv->k_id) {
+              case 0: /* vhostdb.<db> */
+                mod_vhostdb_dbconf_free(cpv->v.v);
+                break;
+              default:
+                break;
             }
         }
     }
 }
-#undef PATCH
 
-/* this function is called at dlopen() time and inits the callbacks */
+static void mod_vhostdb_merge_config_cpv(plugin_config * const pconf, const config_plugin_value_t * const cpv) {
+    switch (cpv->k_id) { /* index into static config_plugin_keys_t cpk[] */
+      case 0: /* vhostdb.<db> */
+        if (cpv->vtype == T_CONFIG_LOCAL)
+            pconf->vdata = cpv->v.v;
+        break;
+      default:/* should not happen */
+        return;
+    }
+}
+
+static void mod_vhostdb_merge_config(plugin_config * const pconf, const config_plugin_value_t *cpv) {
+    do {
+        mod_vhostdb_merge_config_cpv(pconf, cpv);
+    } while ((++cpv)->k_id != -1);
+}
+
+static void mod_vhostdb_patch_config(request_st * const r, plugin_data * const p) {
+    p->conf = p->defaults; /* copy small struct instead of memcpy() */
+    /*memcpy(&p->conf, &p->defaults, sizeof(plugin_config));*/
+    for (int i = 1, used = p->nconfig; i < used; ++i) {
+        if (config_check_cond(r, (uint32_t)p->cvlist[i].k_id))
+            mod_vhostdb_merge_config(&p->conf,p->cvlist + p->cvlist[i].v.u2[0]);
+    }
+}
+
+SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
+    static const config_plugin_keys_t cpk[] = {
+      { CONST_STR_LEN("vhostdb.dbi"),
+        T_CONFIG_ARRAY_KVANY,
+        T_CONFIG_SCOPE_CONNECTION }
+     ,{ NULL, 0,
+        T_CONFIG_UNSET,
+        T_CONFIG_SCOPE_UNSET }
+    };
+
+    plugin_data * const p = p_d;
+    if (!config_plugin_values_init(srv, p, cpk, "mod_vhostdb_dbi"))
+        return HANDLER_ERROR;
+
+    /* process and validate config directives
+     * (init i to 0 if global context; to 1 to skip empty global context) */
+    for (int i = !p->cvlist[0].v.u2[1]; i < p->nconfig; ++i) {
+        config_plugin_value_t *cpv = p->cvlist + p->cvlist[i].v.u2[0];
+        for (; -1 != cpv->k_id; ++cpv) {
+            switch (cpv->k_id) {
+              case 0: /* vhostdb.<db> */
+                if (cpv->v.a->used) {
+                    if (0 != mod_vhostdb_dbconf_setup(srv, cpv->v.a, &cpv->v.v))
+                        return HANDLER_ERROR;
+                    if (NULL != cpv->v.v)
+                        cpv->vtype = T_CONFIG_LOCAL;
+                }
+                break;
+              default:/* should not happen */
+                break;
+            }
+        }
+    }
+
+    /* initialize p->defaults from global config context */
+    if (p->nconfig > 0 && p->cvlist->v.u2[1]) {
+        const config_plugin_value_t *cpv = p->cvlist + p->cvlist->v.u2[0];
+        if (-1 != cpv->k_id)
+            mod_vhostdb_merge_config(&p->defaults, cpv);
+    }
+
+    return HANDLER_GO_ON;
+}
+
+
 int mod_vhostdb_dbi_plugin_init (plugin *p);
 int mod_vhostdb_dbi_plugin_init (plugin *p)
 {
     p->version          = LIGHTTPD_VERSION_ID;
-    p->name             = buffer_init_string("vhostdb_dbi");
+    p->name             = "vhostdb_dbi";
 
     p->init             = mod_vhostdb_init;
     p->cleanup          = mod_vhostdb_cleanup;
