@@ -529,7 +529,7 @@ static int network_socket_activation_from_env(server *srv, network_socket_config
     char *listen_fds = getenv("LISTEN_FDS");
     pid_t lpid = listen_pid ? (pid_t)strtoul(listen_pid,NULL,10) : 0;
     int nfds = listen_fds ? atoi(listen_fds) : 0;
-    int rc = (lpid == getpid() && nfds > 0)
+    int rc = (lpid == getpid() && nfds > 0 && nfds < 5000)
       ? network_socket_activation_nfds(srv, s, nfds)
       : 0;
     unsetenv("LISTEN_PID");
@@ -686,6 +686,8 @@ int network_init(server *srv, int stdin_fd) {
                 force_assert(NULL != srv_socket);
                 memcpy(srv_socket, srv->srv_sockets_inherited.ptr[i],
                        sizeof(server_socket));
+                srv_socket->srv_token =
+                  buffer_init_buffer(srv_socket->srv_token);
                 network_srv_sockets_append(srv, srv_socket);
             }
         }
