@@ -359,7 +359,7 @@ static int server_oneshot_read_cq(connection *con, chunkqueue *cq, off_t max_byt
      * read interest after FDEVENT_IN interest was paused for other reasons */
 
     const int events = fdevent_fdnode_interest(oneshot_fdn);
-    int n = con->is_readable ? 0 : FDEVENT_IN;
+    int n = con->is_readable > 0 ? 0 : FDEVENT_IN;
     if (events & FDEVENT_RDHUP)
         n |= FDEVENT_RDHUP;
     fdevent_fdnode_event_set(con->srv->ev, oneshot_fdn, n);
@@ -555,6 +555,11 @@ static void show_features (void) {
 #else
       "\t- zlib support\n"
 #endif
+#if defined HAVE_ZSTD_H && defined HAVE_ZSTD
+      "\t+ zstd support\n"
+#else
+      "\t- zstd support\n"
+#endif
 #if defined HAVE_BZLIB_H && defined HAVE_LIBBZ2
       "\t+ bzip2 support\n"
 #else
@@ -788,8 +793,8 @@ static int server_graceful_state_bg (server *srv) {
     /* check if server should fork and background (bg) itself
      * to continue processing requests already in progress */
     data_unset * const du =
-      array_get_element_klen(srv->srvconf.feature_flags,
-                             CONST_STR_LEN("server.graceful-restart-bg"));
+      array_get_data_unset(srv->srvconf.feature_flags,
+                           CONST_STR_LEN("server.graceful-restart-bg"));
     if (!config_plugin_value_tobool(du, 0)) return 0;
 
     /*(set flag to false to avoid repeating)*/
@@ -926,7 +931,7 @@ static void server_graceful_state (server *srv) {
 
     if (!srv_shutdown) {
         if (0 == srv->graceful_expire_ts && srv->srvconf.feature_flags) {
-            data_unset * const du =
+            const data_unset * const du =
               array_get_element_klen(srv->srvconf.feature_flags,
                 CONST_STR_LEN("server.graceful-shutdown-timeout"));
             srv->graceful_expire_ts = config_plugin_value_to_int32(du, 0);
