@@ -3,7 +3,7 @@
 #include "stat_cache.h"
 #include "log.h"
 #include "fdevent.h"
-#include "etag.h"
+#include "http_etag.h"
 #include "algo_splaytree.h"
 
 #include <sys/types.h>
@@ -498,8 +498,7 @@ static void stat_cache_handle_fdevent_fn(stat_cache_fam * const scf, fam_dir_ent
                 /* temporarily append filename to dir in fam_dir->name to
                  * construct path, then delete stat_cache entry (if any)*/
                 len = buffer_string_length(n);
-                buffer_append_string_len(n, CONST_STR_LEN("/"));
-                buffer_append_string_len(n, fn, fnlen);
+                buffer_append_path_len(n, fn, fnlen);
                 /* (alternatively, could chose to stat() and update)*/
                 stat_cache_invalidate_entry(CONST_BUF_LEN(n));
 
@@ -1059,7 +1058,7 @@ const buffer * stat_cache_etag_get(stat_cache_entry *sce, int flags) {
 
     if (S_ISREG(sce->st.st_mode) || S_ISDIR(sce->st.st_mode)) {
         if (0 == flags) return NULL;
-        etag_create(&sce->etag, &sce->st, flags);
+        http_etag_create(&sce->etag, &sce->st, flags);
         return &sce->etag;
     }
 
