@@ -16,7 +16,7 @@ static void run_burl_normalize (buffer *psrc, buffer *ptmp, int flags, int line,
                 __FILE__, line, __func__+4, in, psrc->ptr);
     }
     else {
-        if (buffer_is_equal_string(psrc, out, out_len)) return;
+        if (buffer_eq_slen(psrc, out, out_len)) return;
         fprintf(stderr,
                 "%s.%d: %s('%s') failed: expected '%s', got '%s'\n",
                 __FILE__, line, __func__+4, in, out, psrc->ptr);
@@ -141,6 +141,7 @@ static void test_burl_normalize (void) {
     run_burl_normalize(psrc, ptmp, flags, __LINE__, CONST_STR_LEN("/a/b?c=d%20e"), CONST_STR_LEN("/a/b?c=d+e"));
     flags &= ~HTTP_PARSEOPT_URL_NORMALIZE_QUERY_20_PLUS;
 
+    UNUSED(flags);
     buffer_free(psrc);
     buffer_free(ptmp);
 }
