@@ -1,7 +1,13 @@
 #include "first.h"
 
 #include "vector.h"
-#include "base.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+void vector_free(void *data) { free(data); }
+
+void *vector_malloc(size_t sz) { return malloc(sz); }
 
 void *vector_realloc(void *data, size_t elem_size, size_t size, size_t used) {
 	const size_t total_size = elem_size * size;
